#ifndef PEW_HITTABLE_TRIANGLE
#define PEW_HITTABLE_TRIANGLE

#include "pew/hittable/hittable.hpp"
#include "pew/material/material.hpp"
#include "pew/vector/common.hpp"
namespace pew {
    struct Triangle final : public Hittable {
        vec3 v0;
        vec3 v1;
        vec3 v2;

        Triangle() = default;
        Triangle(vec3 a, vec3 b, vec3 c, std::shared_ptr<Material> material) :
            Hittable(material), v0(a), v1(b), v2(c) {}

        std::optional<Hit>
        hit(const Ray& ray) const override {
            static auto e1 = v1 - v0;
            static auto e2 = v2 - v0;

            auto h = ray.getDirection().cross(e2);
            auto a = e1.dot(h);

            if (a > -0.00001 && a < 0.00001) return std::nullopt;

            auto f = 1 / a;
            auto s = ray.getOrigin() - v0;
            auto u = f * s.dot(h);

            if (u < 0.0 || u > 1.0) return std::nullopt;

            auto q = s.cross(e1);
            auto v = f * ray.getDirection().dot(q);

            if (v < 0.0 || u + v > 1.0) return std::nullopt;

            auto t = f * e2.dot(q);

            if (t < ray.getRange().first || t > ray.getRange().second) return std::nullopt;

            Hit hit;
            hit.t   = t;
            hit.ray = ray;
            hit.setFaceNormal(e1.cross(e2).unit());
            hit.material = material;

            return hit;
        }
    };
} // namespace pew

#endif